import pygame
from pygame.locals import *

import resource
import display
from helper import *

class Render:
	[Flat, Tall] = range(2)

class Animations:
	[Walking, Attacking, Pitted] = range(3)

class Directions:
	[UpLeft, DownLeft, UpRight, DownRight] = range(4)

class Cursor(pygame.sprite.Sprite):
	def __init__(self):
		self.image = resource.get_image('Cursor')
		self.offset = (-self.image.get_width(), -self.image.get_height())
		self.update_position()
		self.layer = 9000
		pygame.sprite.Sprite.__init__(self)
	
	def update_position(self):
		self.rect = pygame.Rect(pygame.mouse.get_pos(), (0,0)).move(self.offset)

class MobSprite(pygame.sprite.Sprite):
	def __init__(self, name):
		self.cycle = 2
		self.speed = 2
		self.idle = True
		self.dest = None
		self.rect = None
		self.delay = 0
		self.anim_wait = 0
		self.anim_speed = 4
		self.elevation = 0
		self.image = resource.get_image(name)
		self.anims = resource.get_animations(name)
		self.anim = Animations.Walking
		self.dir = Directions.DownLeft
		pygame.sprite.Sprite.__init__(self)
	
	def set_location(self, s_pos):
		dest = None
		self.loc = tup2rect(s_pos)
		self.fix_rect()
	
	def fix_rect(self):
		display.get_screen().sprites.change_layer(self, self.loc.top * max(self.elevation * 20, 1))
		self.rect = pygame.Rect(self.loc.left, self.loc.top - self.image.get_height() + TILE_PIXEL_HEIGHT - self.elevation, 0, 0)
	
	def move_to(self, s_pos):
		self.dest = tup2rect(s_pos)
		self.delay = 0
		self.idle = False
		self.dir = self.direction_toward(self.dest)
		self.anim = Animations.Walking
		self.set_frame(0)
	
	def do_move(self):
		self.delay += 1
		if self.delay >= self.cycle * self.mob.get_slow_factor():
			x, y = 0, 0
			if self.dest.top > self.loc.top:
				y = min(self.dest.top - self.loc.top, self.speed / 2)
			elif self.dest.top < self.loc.top:
				y = max(self.dest.top - self.loc.top, -self.speed / 2)
				
			if self.dest.left > self.loc.left:
				x = min(self.dest.left - self.loc.left, self.speed)
			elif self.dest.left < self.loc.left:
				x = max(self.dest.left - self.loc.left, -self.speed)
			
			self.loc = self.loc.move(x, y)
			self.fix_rect()
			if (self.loc == self.dest):
				self.idle = True
				self.dest = None
			else:
				self.delay = 0
	
	def show_attack(self, target):
		self.anim = Animations.Attacking
		self.dir = self.direction_toward(target.sprite.loc)
		self.set_frame(0)
		self.anim_wait = 30
		self.idle = False
	
	def pitted(self):
		self.anim = Animations.Pitted
		self.set_frame(0)
		if self.dest != None:
			self.loc = pygame.Rect(self.dest)
			self.dest = None
			self.fix_rect()
		self.anim_wait = 400
		self.idle = False
	
	def update(self, tick):
		if (tick % self.anim_speed) == 0:
			self.set_frame(tick)
			
		if self.dest != None:
			self.do_move()
		elif self.anim_wait > 0:
			self.anim_wait -= 1
		elif self.anim_wait <= 0:
			self.idle = True
		
		if self.idle:
			self.anim = Animations.Walking # should be standing once that is added
	
	def set_frame(self, tick):
		self.image = self.anim_frame(self.anim, self.dir, tick / self.anim_speed)
	
	def anim_frame(self, animation, direction, frame):
		frame = frame % len(self.anims[animation])
		return self.anims[animation][frame][self.dir]
	
	def direction_toward(self, s_target):
		dx = self.loc.left - s_target.left
		dy = self.loc.top - s_target.top
		if dx > 0 and dy > 0:
			return Directions.UpLeft
		elif dx < 0 and dy < 0:
			return Directions.DownRight
		elif dx < 0 and dy > 0:
			return Directions.UpRight
		else:
			return Directions.DownLeft

class TerrainSprite(pygame.sprite.Sprite):
	def __init__(self, name):
		self.cycle = 2
		self.speed = 2
		self.idle = True
		self.image = resource.get_image(name)
		pygame.sprite.Sprite.__init__(self)
	
	def set_location(self, s_pos):
		self.loc = tup2rect(s_pos)
		self.fix_rect()

	def fix_rect(self):
		if self.render == Render.Flat:
			display.get_screen().sprites.change_layer(self, -9001)
		else:
			display.get_screen().sprites.change_layer(self, self.loc.top)
		self.rect = pygame.Rect(self.loc.left, self.loc.top - self.image.get_height() + TILE_PIXEL_HEIGHT, 0, 0)
	
	def change_image(self, name):
		self.image = resource.get_image(name)
	
	def update(self, tick):
		pass


class ProjectileSprite(pygame.sprite.Sprite):
	def __init__(self, name, el):
		self.rect = None
		self.dest = None
		self.cycle = 1
		self.speed = 8
		self.zpos = 0
		self.zvel = 0
		self.zacc = 0
		self.delay = 0
		self.elevation = el
		self.image = resource.get_image(name)
		pygame.sprite.Sprite.__init__(self)	

	def set_location(self, s_pos):
		self.loc = tup2rect(s_pos)
		self.fix_rect()
	
	def fix_rect(self):
		display.get_screen().sprites.change_layer(self, self.loc.top * max(self.zpos, 1))
		self.rect = pygame.Rect(self.loc.left, self.loc.top - self.image.get_height() + TILE_PIXEL_HEIGHT - self.elevation, 0, 0)

	def update(self, tick):	
		self.do_move()

	def do_move(self):
		self.delay += 1
		if self.delay >= self.cycle:
			x = self.dest.left - self.loc.left
			y = self.dest.top - self.loc.top
			manhattan = abs(x) + abs(y)
			
			cycs = (manhattan / self.speed) + 1
			
			if manhattan > self.speed:
				scale = float(self.speed) / float(manhattan)
				x *= scale
				y *= scale
			else:
				self.proj.impact()
				return
			
			
			self.loc = self.loc.move(x, y)
			self.elevation = max(0, self.elevation - self.elevation / cycs)
			self.fix_rect()
			if (self.loc == self.dest):
				self.idle = True
				self.dest = None
			else:
				self.delay = 0